<?php namespace EwCacheInvalidator;

/**
 * Class PageDebug
 * @package EwCacheInvalidator
 */
class PageDebug {

    /**
     * PageDebug constructor.
     */
    public function __construct() {

        add_action('admin_enqueue_scripts', function() {

            if(isset($_GET['page']) && $_GET['page'] === 'ew-cache-invalidator-debug') {
                wp_enqueue_script('ew-cache-invalidator-settings-page', plugins_url('../src/assets/js/admin.js', __FILE__));
                wp_register_style('ew-cache-invalidator-admin-style', plugins_url('../src/assets/css/admin.css', __FILE__));
                wp_enqueue_style('ew-cache-invalidator-admin-style' );
            }
        });
    }

    /**
     * Render the admin page.
     */
    public function renderAdminPage() {

        $dataStorage = new DataStorage();
        $storedQueryArr = $dataStorage->getAllItems();

        $formattedArr = [];
        $queryCount = 0;
        foreach($storedQueryArr as $queryItem) {

            $permalink = '';
            if(isset($queryItem->page_id)) {
                $permalink = get_permalink($queryItem->page_id);
            }
            else if(isset($queryItem->page_url)) {
                $permalink = $queryItem->page_url;
            }
            else {
                continue;
            }

            if(!array_key_exists($permalink, $formattedArr)) {
                $formattedArr[$permalink] = [];
            }

            $formattedArr[$permalink][] = $queryItem;
            $queryCount++;
        }

        print '<h1>Data used for cache invalidation</h1>';
        print '<p class="description">There is <strong>' . $queryCount . '</strong> queries saved for <strong>' . count($formattedArr) . '</strong> pages.</p>';
        print '<p class="description">Click a page URL to see which queries are saved for the given page.</p>';

        if(count($storedQueryArr) === 0) {
            print '<p>No data found.</p>';
        }
        else {
            ksort($formattedArr);
            foreach($formattedArr as $permalink => $queryItemArr) {

                print '<div class="page-section">';
                    print '<h3 class="page-header">' . $permalink . ' (' . count($queryItemArr) . ')</h3>';

                    print '<table class="page-table widefat fixed" style="margin-bottom: 50px;" cellspacing="0">';
                        print '<thead>';
                            print '<th colspan="2"><strong>OC Query</strong></th>';
                        print '</thead>';

                        $count = 0;
                        foreach($queryItemArr as $queryItem) {

                            $rowClass = ($count % 2 == 1) ? "alternate" : "";

                            print '<tr class="' . $rowClass . '">';
                                print '<td colspan="2">' . $queryItem->oc_query . '</td>';
                            print '</tr>';

                            print '<tr class="' . $rowClass . '">';
                                print '<td><strong>Type:</strong> ' . $queryItem->type . '</td>';
                                print '<td><strong>Stream ID:</strong> ' . $queryItem->listener_id . '</td>';
                            print '</tr>';

                            $count++;
                        }
                    print '</table>';
                print '</div>';
            }
        }
    }
}