<?php namespace EwCacheInvalidator;

use EwCacheInvalidator\PageQueries;
use EwCacheInvalidator\DataStorage;
use EwCacheInvalidator\Lcc;

/**
 * Class PageSettings
 * @package EwCacheInvalidator
 */
class PageSettings {

    private static $query_suffix_option_name = 'ew_cache_invalidator_query_suffix';
    private static $query_streamer_url_option_name = 'ew_cache_invalidator_query_streamer_url';
    private static $query_streamer_username_option_name = 'ew_cache_invalidator_query_streamer_username';
    private static $query_streamer_password_option_name = 'ew_cache_invalidator_query_streamer_password';
    private static $event_handling_option_name = 'ew_cache_invalidator_event_handling';

    /**
     * PageSettings constructor.
     */
    public function __construct() {

        add_action('admin_enqueue_scripts', function() {

            if(isset($_GET['page']) && $_GET['page'] === 'ew-cache-invalidator') {
                wp_enqueue_script('ew-cache-invalidator-settings-page', plugins_url('../src/assets/js/admin.js', __FILE__));
                wp_register_style('ew-cache-invalidator-admin-style', plugins_url('../src/assets/css/admin.css', __FILE__));
                wp_enqueue_style('ew-cache-invalidator-admin-style' );
            }
        });
    }

    /**
     * Render the admin page.
     */
    public function renderAdminPage() {

        print '<h1>Cache invalidation settings</h1>';

        print '<div class="admin-section">';

            print '<h3>Query Streamer</h3>';

            print '<p><strong>URL</strong></p>';
            print '<p><input type="text" value="' . PageSettings::getQueryStreamerUrl() . '" class="ew-cache-settings-text-input" id="ew-cache-query-streamer-url" /></p>';
            print '<p class="description"><strong>Required:</strong> set the URL to the Query Streamer.</p>';
            print '<p class="description">Example URL: https://querystreamer.lcc.infomaker.io/v1/stream-provider/<strong>&lt;streamProviderId&gt;</strong>/ (should include Stream Provider ID).</p>';

            print '<p><strong>Username</strong></p>';
            print '<p><input type="text" value="' . PageSettings::getQueryStreamerUsername() . '" class="ew-cache-settings-text-input" id="ew-cache-query-streamer-username" /></p>';

            print '<p><strong>Password</strong></p>';
            print '<p><input type="password" value="' . PageSettings::getQueryStreamerPassword() . '" class="ew-cache-settings-text-input" id="ew-cache-query-streamer-password" /></p>';

            print '<p><input type="button" value="Connect to query streamer" class="button button-primary" id="set-query-streamer-btn" /></p>';
            print '<p class="progress-indicator-set-query-streamer progress-indicator"></p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Query suffix</h3>';
            print '<p><textarea type="button" id="set-query-suffix-textarea">' . PageSettings::getQuerySuffix() . '</textarea></p>';
            print '<p><input type="button" value="Set query suffix" class="button button-primary" id="set-query-suffix-btn" /></p>';
            print '<p class="progress-indicator-set-query-suffix progress-indicator"></p>';
            print '<p class="description">Optional, set a query suffix to be appended to all queries registered in LCC. Example: "AND group:Paper1"</p>';
            print '<p class="description">Changes will only apply to queries registered after a change is made.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Run query lookup</h3>';
            print '<p><input type="button" value="Find all queries" class="button button-primary" id="reset-all-queries-btn" /></p>';
            print '<p class="progress-indicator-reset-all-queries progress-indicator"></p>';
            print '<p class="description">This action removes all existing queries before adding.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Delete all queries</h3>';
            print '<p><input type="button" value="Delete queries" class="button button-primary" id="delete-all-queries-btn" /></p>';
            print '<p class="progress-indicator-delete-all-queries progress-indicator"></p>';
            print '<p class="description">Delete all saved queries, listeners will be deregistered as well.</p>';
        print '</div>';

        $event_checked = PageSettings::getEventHandling() === true ? 'checked="checked"' : '';
        print '<div class="admin-section">';
            print '<h3>Event handling</h3>';
            print '<p>Enabled: <input type="checkbox" id="event-handling-checkbox" ' . $event_checked . ' /><span class="progress-indicator-event-handling progress-indicator"></span></p>';
            print '<p class="description">Enable or disable event handling, if on the plugin will listen to events and update the queries towards LCC.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Query Streamer connection status</h3>';
            print '<p class="ew-cache-invalidator-query-streamer-status"></p>';
            print '<p class="description">For this plugin to work the connection to the Query Streamer must be ok.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Invalidation endpoint status</h3>';
            print '<p class="ew-cache-invalidator-endpoint-status">Checking endpoint status.. <span class="ew-cache-spinner"></span></p>';
            print '<p class="description">For this plugin to work, the environment has to support the <strong>"/ledge-invalidation"</strong> path.</p>';
            print '<p class="description">If it does not work, please look up plugin instructions on how to enable this.</p>';
        print '</div>';
    }

    /**
     * Save query suffix via ajax.
     */
    public static function ajaxSaveQuerySuffix() {

        $status = false;
        if(isset($_POST['suffix'])) {
            $status = self::saveQuerySuffix($_POST['suffix']);
        }

        wp_send_json([
            'status' =>  $status
        ]);

        die();
    }

    /**
     * Save query stream data via ajax.
     */
    public static function ajaxSaveQueryStream() {

        $status = false;
        if(isset($_POST['querystreamer_url'], $_POST['querystreamer_username'], $_POST['querystreamer_password'])) {
            $status = self::saveQueryStreamer($_POST['querystreamer_url'], $_POST['querystreamer_username'], $_POST['querystreamer_password']);
        }

        wp_send_json([
           'status' => $status
        ]);

        die();
    }

    /**
     * Reset all queries via ajax.
     */
    public static function ajaxResetAllQueries() {

        wp_send_json([
            'status' =>  self::resetAllQueries()
        ]);

        die();
    }

    /**
     * Delete all queries via ajax.
     */
    public static function ajaxDeleteAllQueries() {

        $dataStorage = new DataStorage();
        $lcc = new Lcc();

        $listeners = $dataStorage->getAllItems();
        foreach($listeners as $listener) {
            $lcc->deregisterListener($listener->listener_id);
        }

        wp_send_json([
            'status' => $dataStorage->purgeTable()
        ]);

        die();
    }

    /**
     * Check invalidation endpoint status via ajax.
     */
    public static function ajaxCheckEndpointStatus() {

        wp_send_json([
            'status' => self::checkEndpointStatus()
        ]);

        die();
    }

    /**
     * Check the Query Streamer status via ajax.
     */
    public static function ajaxCheckQueryStreamStatus() {

        wp_send_json([
            'status' => self::checkQueryStreamStatus()
        ]);

        die();
    }

    public static function ajaxSaveEventHandling() {

        $status = false;
        if(isset($_POST['eventHandling'])) {
            $value = $_POST['eventHandling'] === 'true' ? true : false;
            $status = self::saveEventHandling($value);
        }

        wp_send_json([
            'status' =>  $status
        ]);

        die();
    }

    /**
     * Get query suffix.
     *
     * @return mixed
     */
    public static function getQuerySuffix() {
        return get_option(self::$query_suffix_option_name, '');
    }

    /**
     * Save query suffix.
     *
     * @param $suffix
     * @return bool
     */
    private static function saveQuerySuffix($suffix) {
        return update_option(self::$query_suffix_option_name, $suffix);
    }

    /**
     * Get Query Streamer url.
     *
     * @return mixed
     */
    public static function getQueryStreamerUrl() {
        return get_option(self::$query_streamer_url_option_name, '');
    }

    /**
     * Save Query Streamer connection data.
     *
     * @param $url
     * @param $username
     * @param $password
     * @return bool
     */
    private static function saveQueryStreamer($url, $username, $password) {

        $url_result = update_option(self::$query_streamer_url_option_name, $url);
        $username_result = update_option(self::$query_streamer_username_option_name, $username);
        $password_result = update_option(self::$query_streamer_password_option_name, $password);

        if($url_result === false || $username_result === false || $password_result === false) {
            return false;
        }

        return true;
    }

    /**
     * Get Query Streamer username.
     *
     * @return mixed
     */
    public static function getQueryStreamerUsername() {
        return get_option(self::$query_streamer_username_option_name, '');
    }

    /**
     * Get Query Streamer password.
     *
     * @return mixed
     */
    public static function getQueryStreamerPassword() {
        return get_option(self::$query_streamer_password_option_name, '');
    }

    /**
     * Get event handling status.
     *
     * @return mixed
     */
    public static function getEventHandling() {
        return get_option(self::$event_handling_option_name, false);
    }

    /**
     * Save event handling status.
     *
     * @param $value
     * @return bool
     */
    private static function saveEventHandling($value) {
        return update_option(self::$event_handling_option_name, $value);
    }

    /**
     * Reset all queries.
     *
     * @return bool
     */
    private static function resetAllQueries() {

        $pageQueries = new PageQueries();
        $dataStorage = new DataStorage();
        $lcc = new Lcc();

        $listeners = $dataStorage->getAllItems();
        foreach($listeners as $listener) {
            $lcc->deregisterListener($listener->listener_id);
        }

        $success = $dataStorage->purgeTable();

        if($success) {

            $queryArr = $pageQueries->getAllQueries();

            foreach($queryArr as $query) {

                if(is_array($query) && count($query) >= 3) {

                    $listener_id = $lcc->registerListener($query[0]);
                    if($listener_id !== false) {
                        $dataStorage->saveItem(new QueryItem($query[0], $query[1], $query[2], $listener_id));
                    }
                }
                else if(is_array($query) && count($query) === 2) {
                    $listener_id = $lcc->registerListener($query[0]);
                    if($listener_id !== false) {
                        $dataStorage->saveItem(new QueryItem($query[0], $query[1], 'custom', $listener_id));
                    }
                }

            }

            return true;
        }

        return false;
    }

    /**
     * Get cache invalidation endpoint status.
     *
     * @return bool
     */
    public static function checkEndpointStatus() {

        $response = wp_remote_post(get_site_url() . '/ledge-invalidate/', [
            'user-agent' => 'ew-ledge-invalidation',
            'sslverify' => false,
            'body' => [
                'testing' => 'true'
            ]
        ]);

        if(!is_wp_error($response) && $response['response']['code'] === 200) {
            return true;
        }

        return false;
    }

    /**
     * Check connection status towards the query streamer.
     *
     * @return bool
     */
    public static function checkQueryStreamStatus() {

        $lcc = new Lcc();
        return $lcc->getStatus();
    }

    /**
     * Check if we should listen to events.
     *
     * @return bool
     */
    public static function checkQueryEventsActive() {
        return self::getEventHandling();
    }
}